using System;
using FlowEngine;

public class Client
{
	
	public Client()
	{
		m_messageActionFactory = null;
		m_processFactory       = null;
	}
	
	
	
	private static int run(string[] args, Ice.Communicator communicator)
    {
        Ice.Properties properties = communicator.getProperties();        
        string proxy;
        
        // ------- initialize process factory -------------------------------------------
        proxy = properties.getProperty("ProcessFactory.Proxy");
        if (proxy.Length == 0)
        { 
        	Console.Error.WriteLine("property not set");           
        }
        else
        {
	        m_processFactory = ProcessFactoryPrxHelper.checkedCast( communicator.stringToProxy(proxy) );
	        if(m_processFactory == null)
	        {
	            Console.Error.WriteLine("invalid proxy");
	        }
        }
        
        
        // ------- initialize message action factory -------------------------------------------
        proxy = properties.getProperty("MessageActionFactory.Proxy");
        if(proxy.Length == 0)
        {
            Console.Error.WriteLine("property not set");            
        }
        else
        {
	        m_messageActionFactory = AbstractActionFactoryPrxHelper.checkedCast( communicator.stringToProxy(proxy) );
	        if(m_messageActionFactory == null)
	        {
	            Console.Error.WriteLine("invalid proxy");
	        }
        }
        
        
        // ------- initialize wait for file action factory -------------------------------------------
        proxy = properties.getProperty("WaitForFileActionFactory.Proxy");
        if(proxy.Length == 0)
        {
            Console.Error.WriteLine("property not set");            
        }
        else
        {
	        m_waitforfileActionFactory = AbstractActionFactoryPrxHelper.checkedCast( communicator.stringToProxy(proxy) );
	        if(m_waitforfileActionFactory == null)
	        {
	            Console.Error.WriteLine("invalid proxy");
	        }
        }
        
        
        // ------- initialize start action factory -------------------------------------------
        proxy = properties.getProperty("StartActionFactory.Proxy");
        if (proxy.Length == 0)
        { 
        	Console.Error.WriteLine("property not set");           
        }
        else
        {
	        m_startActionFactory = AbstractActionFactoryPrxHelper.checkedCast( communicator.stringToProxy(proxy) );
	        if(m_startActionFactory == null)
	        {
	            Console.Error.WriteLine("invalid proxy");
	        }
        }
        
        // ------- initialize decision action factory -------------------------------------------
        proxy = properties.getProperty("DecisionActionFactory.Proxy");
        if (proxy.Length == 0)
        { 
        	Console.Error.WriteLine("property not set");           
        }
        else
        {
	        m_decisionActionFactory = AbstractActionFactoryPrxHelper.checkedCast( communicator.stringToProxy(proxy) );
	        if(m_decisionActionFactory == null)
	        {
	            Console.Error.WriteLine("invalid proxy");
	        }
        }
        
        // ------- initialize end action factory -------------------------------------------
        proxy = properties.getProperty("EndActionFactory.Proxy");
        if (proxy.Length == 0)
        { 
        	Console.Error.WriteLine("property not set");           
        }
        else
        {
	        m_endActionFactory = AbstractActionFactoryPrxHelper.checkedCast( communicator.stringToProxy(proxy) );
	        if(m_endActionFactory == null)
	        {
	            Console.Error.WriteLine("invalid proxy");
	        }
        }
        
        ProcessPrx newProcess               = m_processFactory.createProcess();
        AbstractActionPrx message1actionPrx = m_messageActionFactory.createAction();
        AbstractActionPrx waitActionPrx     = m_waitforfileActionFactory.createAction();
        AbstractActionPrx startActionPrx    = m_startActionFactory.createAction();
        AbstractActionPrx endActionPrx      = m_endActionFactory.createAction();
        
        /*
        Global.KeyValueHelper listHelper = new Global.KeyValueHelper();
        listHelper.addItem("FilePath", "c:\\temp\\thisfileshouldexist.txt");
        
        Global.KeyValueList list = listHelper.getList();        
        System.Console.WriteLine("Number of options: " + list.Count);
        */
        Global.KeyValue kv;
        kv.key   = "FilePath";
        kv.value = "c:\\temp\\thisfileshouldexist.txt";
        Global.KeyValueList list = new Global.KeyValueList();
        list.Add(kv);
        
        waitActionPrx.setOptions(list);
        
        int id0 = newProcess.addAction(startActionPrx);
        int id1 = newProcess.addAction(waitActionPrx);
        int id2 = newProcess.addAction(message1actionPrx);
      	int id3 = newProcess.addAction(endActionPrx);
      	
      	newProcess.setEdge(id0, id1);
      	newProcess.setEdge(id1, id2);
      	newProcess.setEdge(id2, id3);
      	
      	newProcess.tempIterate();
        
        return 0;
    }
    
	
	
    public static void Main(string[] args)
    {
        int status = 0;
        Ice.Communicator communicator = null;
        
        try
        {
            Ice.Properties properties = Ice.Util.createProperties();
            properties.load("config.txt");
            communicator = Ice.Util.initializeWithProperties(ref args, properties);
            status       = run(args, communicator);
        }
        catch(System.Exception ex)
        {
	    	Console.Error.WriteLine(ex);
            status = 1;
        }

		if(communicator != null)
		{
		    try
		    {
				communicator.destroy();
		    }
		    catch(System.Exception ex)
		    {
				Console.Error.WriteLine(ex);
				status = 1;
		    }
		}
        
        System.Environment.Exit(status);
    }
    
    private static AbstractActionFactoryPrx m_messageActionFactory;
    private static ProcessFactoryPrx m_processFactory;
    private static AbstractActionFactoryPrx m_startActionFactory;
    private static AbstractActionFactoryPrx m_decisionActionFactory;
    private static AbstractActionFactoryPrx m_endActionFactory;
    private static AbstractActionFactoryPrx m_waitforfileActionFactory;
}
