#ifndef _FLOWENGINE_ICE_
#define _FLOWENGINE_ICE_


#include "global.ice"



module FlowEngine
{
	interface AbstractAction; // forward declaration
	["cs:collection"] sequence<AbstractAction*> AbstractActionPrxList;
	dictionary<int,int> IntMap;
	
	
	interface AbstractAction
	{
		nonmutating string describe();
		
		void setOptions(Global::KeyValueList options);
		
		nonmutating int getId();
		
		void setId(int id);
		
		/**
		 * Performs the respective action
		 * 
		 * @todo might be asynchronous one day
		 */
		["ami"] void perform();
		
		/**
		 * Returns if the action is still working or not
		 */
		nonmutating bool done();
	};
	
	interface AbstractActionFactory
	{
		AbstractAction* createAction();
		
		nonmutating string describe();
	};
	
	interface Process
	{		
		/** 
		 * Returns all proxies of the process
		 */ 
		nonmutating AbstractActionPrxList enumActions();
		
		/**
		 * Returns all start items of the process
		 */
		nonmutating AbstractActionPrxList enumStartActions();
		
		/**
		 * Returns the action which is currently processed
		 */ 
		nonmutating AbstractAction* getCurrentAction();
		
		/**
		 * Returns a proxy of an action within the process
		 */
		nonmutating AbstractAction* findAction(int id);
		
		/**
		 * Returns the edges for the entire process by specifying
		 * a map of start point (id of action1) and end point
		 * (id of action2)
		 */
		nonmutating IntMap getConnectionMap();
		
		int addAction(AbstractAction* action);
		
		void addActionWithId(AbstractAction* action, int id);
		
		nonmutating bool isActive();
		
		void setEdge(int id0, int id1);
		
		nonmutating AbstractActionPrxList getNextActionsById(int id);
		
		nonmutating AbstractActionPrxList getNextActions(AbstractAction* action);
		
		bool tempIterate();
	};
	
	interface ProcessFactory
	{
		Process* createProcess();
	};
};


#endif // !_FLOWENGINE_ICE_