using MessageAction;
using WaitForFileAction;
using ProcessEngine;
using StateActions;


	
public class Server
{
    private static int run(string[] args, Ice.Communicator communicator)
    {
    	try
    	{
	        Ice.ObjectAdapter adapter                       = communicator.createObjectAdapter("processengine");
	        ProcessFactoryI processFactoryObj               = new ProcessFactoryI();
	        
	        StartActionFactoryI startActionFactoryObj       = new StartActionFactoryI();
	        EndActionFactoryI endActionFactoryObj           = new EndActionFactoryI();
	        DecisionActionFactoryI decisionActionFactoryObj = new DecisionActionFactoryI();
	        MessageActionFactoryI msgActionFactoryObj       = new MessageActionFactoryI();
	        WaitForFileActionFactoryI waitActionObj         = new WaitForFileActionFactoryI();        
	        adapter.add(msgActionFactoryObj, Ice.Util.stringToIdentity("messageactionfactory"));
	        adapter.add(waitActionObj, Ice.Util.stringToIdentity("waitforfileactionfactory"));
	        adapter.add(startActionFactoryObj, Ice.Util.stringToIdentity("startactionfactory"));
	        adapter.add(endActionFactoryObj, Ice.Util.stringToIdentity("endactionfactory"));
	        adapter.add(decisionActionFactoryObj, Ice.Util.stringToIdentity("decisionactionfactory"));
	        
	        adapter.add(processFactoryObj, Ice.Util.stringToIdentity("processfactory"));        
	        processFactoryObj.setAdapter(adapter);
	        msgActionFactoryObj.setAdapter(adapter);
	        waitActionObj.setAdapter(adapter);
	        startActionFactoryObj.setAdapter(adapter);
	        decisionActionFactoryObj.setAdapter(adapter);
	        endActionFactoryObj.setAdapter(adapter);
	        
	        adapter.activate();
	        System.Console.WriteLine("The server is ready.");
	        communicator.waitForShutdown();
    	}
    	catch(Ice.Exception ex)
    	{
    		System.Console.Error.WriteLine(ex.ToString());
    	}
        return 0;
    }
    
    
    
    public static void Main(string[] args)
    {
        int status = 0;
        Ice.Communicator communicator = null;
        
        try
        {
            Ice.Properties properties = Ice.Util.createProperties();
            properties.load("config.txt");            
            communicator = Ice.Util.initializeWithProperties(ref args, properties);
            status       = run(args, communicator);
        }
        catch(System.Exception ex)
        {
	   		System.Console.Error.WriteLine(ex);
            status = 1;
        }
        
		if(communicator != null)
		{
		    try
		    {
				communicator.destroy();
		    }
		    catch(System.Exception ex)
		    {
				System.Console.Error.WriteLine(ex);
				status = 1;
		    }
		}
        
        System.Environment.Exit(status);
    }
}
