using System;
using FlowEngine;



namespace ProcessEngine
{
	public class AMI_AbstractAction_performI : AMI_AbstractAction_perform
	{
		public override void ice_exception(Ice.Exception ex)
		{
		} // !ice_exception()
		
		
		
		public override void ice_response()
		{
			System.Console.WriteLine("Action finished.");
		} // !ice_response()
	}
	
	

	
	/**
	 * @todo use mutexes for wrapping around m_rgConnections and m_rgActions	 
	 * @todo use a multimap for outputs of an action
	 */
	public class ProcessI : _ProcessDisp
	{
		public ProcessI()
		{
			m_nCurrentActionId = 1;
			m_rgActions        = new AbstractActionPrxList();
			m_rgConnections    = new FlowEngine.IntMap();
		} // !ProcessI()
		
		
		
		public override FlowEngine.AbstractActionPrxList enumActions(Ice.Current current)
		{
			return m_rgActions;	
		} // !enumActions()
		
		
		
		public override FlowEngine.AbstractActionPrxList enumStartActions(Ice.Current current)
		{
			AbstractActionPrxList retval = new AbstractActionPrxList();
			
			foreach (FlowEngine.AbstractActionPrx prx in m_rgActions)
			{
				if (prx.describe() == "StartAction")
				{
					retval.Add(prx);
				}				
			}			
			return retval;
		} // !enumStartActions()
		
		
		
		public override FlowEngine.AbstractActionPrx getCurrentAction(Ice.Current current)
		{
			if (m_nCurrentActionId == 0) // quick out
			{
				return null;
			}
			
			foreach (FlowEngine.AbstractActionPrx prx in m_rgActions)
			{
				if ((prx != null) && (prx.getId() == m_nCurrentActionId))
				{
					return prx;
				}
			}
			
			return null;
		} // !getCurrentAction()
		
		
		
		public override FlowEngine.AbstractActionPrx findAction(int id, Ice.Current current)
		{
			foreach (FlowEngine.AbstractActionPrx prx in m_rgActions)
			{
				if ((prx != null) && (prx.getId() == id))
				{
					return prx;
				}
			}
			return null;
		} // !findAction()
		
		
		
		public override FlowEngine.IntMap getConnectionMap(Ice.Current current)
		{
			return m_rgConnections;
		} // !getConnectionMap()
		
		
		
		public override int addAction(AbstractActionPrx action, Ice.Current current)
		{
			if (action == null)
			{
				System.Console.WriteLine("Invalid action!");
				return 0;
			}
			
			/**
			 * @todo search for empty id 
			 */
			action.setId(++m_nCurrentActionId);
			m_rgActions.Add(action);
			
			return m_nCurrentActionId;
		} // !addAction()
		
		
		
		public override void addActionWithId(AbstractActionPrx action, int id, Ice.Current current)
		{
			if (action == null)
			{
				System.Console.WriteLine("Invalid action!");
				return;
			}
			
			/**
			 * @todo search for id conflict and throw an exception
			 */
			action.setId(id);
			m_rgActions.Add(action);			
		} // !addActionWithId()
		
		
		
		public override bool isActive(Ice.Current current)
		{
			return (m_nCurrentActionId != 0);			
		} // !isActive()
		
		
		public override void setEdge(int id0, int id1, Ice.Current current)
		{
			m_rgConnections.Add(id0, id1);
		} // !setEdge()
		
		
		
		public override AbstractActionPrxList getNextActionsById(int id, Ice.Current current)
		{
			AbstractActionPrxList retval = new AbstractActionPrxList();
			if (!m_rgConnections.Contains(id))
			{				
				return retval;
			}
			
			retval.Add(findAction(m_rgConnections[id], current));
			return retval;
		} // !getNextActionsById()
		
		
		
		public override AbstractActionPrxList getNextActions(AbstractActionPrx prx, Ice.Current current)
		{
			if (prx == null)
			{
				AbstractActionPrxList retval = new AbstractActionPrxList();
				return retval;
			}
			return getNextActionsById(prx.getId());
		} // !getNextActions()
		
		
		
		public override bool tempIterate(Ice.Current current)
		{
			AbstractActionPrxList startActions = enumStartActions(current);
			if (startActions.Count == 0)
			{
				System.Console.WriteLine("No start action");
				return false;
			}
			
			AbstractActionPrx action = startActions[0];
			
			
			// asynchronous call
			AMI_AbstractAction_performI cb = new AMI_AbstractAction_performI();
			action.perform_async(cb);			
			
			// process the workflow
			while (true)
			{	
				System.Console.WriteLine("loop");
				
				if (action == null)
				{
					break;
				}
				
				// wait a little bit until the next check if we are done
				System.Threading.Thread.Sleep(50);	
				
				// check what to do
				if (action.done())
				{
					if (action.describe() == "EndAction")  // <---- are we done?
					{
						System.Console.WriteLine("--- action successfully finished.");
						return true;
					}
					else   // <----- otherwise, get the next item
					{							
						AbstractActionPrxList nextActions = getNextActions(action);
												
						if ((nextActions.Count == 0) || (nextActions[0] == null))
						{
							break;
						}
						
						action = nextActions[0];
														
						System.Console.WriteLine("Before call");
						
						cb = new AMI_AbstractAction_performI();
						action.perform_async(cb);
						
						System.Console.WriteLine("After call");
					}
				}			
			} // !while(true)
			
			return false;
		} // !tempIterate()
		
		
		private int m_nCurrentActionId;
		private AbstractActionPrxList m_rgActions;
		private FlowEngine.IntMap m_rgConnections;
	}
	
	
	
	
	
	
	public class ProcessFactoryI : _ProcessFactoryDisp
	{
		public override FlowEngine.ProcessPrx createProcess(Ice.Current current)
		{
			if (m_adapter == null)
			{	
				/** 
				 * @todo add debugging info
				 */
				return null;
			}
			
			m_enumerator++;			
			ProcessI obj = new ProcessI();
			Ice.ObjectPrx prx  = m_adapter.add(obj, Ice.Util.stringToIdentity("process" + m_enumerator.ToString("D8")));
	    	return FlowEngine.ProcessPrxHelper.uncheckedCast(prx);
		}
	
		
	
		public void setAdapter(Ice.ObjectAdapter adapter)
		{
			m_adapter = adapter;
		}
		
		
		
		private Ice.ObjectAdapter m_adapter;
		private int m_enumerator;
	}
}
